[DEFAULT]
network_type: {{ sahara_network_type }}
network_private_name: {{ private_network_name }}
network_public_name: {{ public_network_name }}
plugin_image: {{ sahara_image_name }}
ci_flavor_id: {{ sahara_flavor_small }}
cluster_name: testc
is_transient: {{ sahara_cluster_transient }}
auto_security_group: {{ sahara_auto_security_group }}
{% if sahara_enable_s3 -%}
s3_accesskey: {{ sahara_s3_accesskey }}
s3_secretkey: {{ sahara_s3_secretkey }}
s3_endpoint: {{ sahara_s3_endpoint }}
s3_endpoint_ssl: {{ sahara_s3_endpoint_ssl }}
s3_bucket_path: {{ sahara_s3_bucket_path }}
{% endif -%}
